import marshmallow as ma
from marshmallow import validate as ma_validate
from oarepo_runtime.i18n.ui_schema import (
    I18nStrUIField,
    MultilingualLocalizedUIField,
    MultilingualUIField,
)
from oarepo_runtime.ui import marshmallow as l10n
from oarepo_runtime.ui.marshmallow import InvenioUISchema

from nr_metadata.common.services.records.ui_schema_datatypes import (
    NRAccessRightsVocabularyUISchema,
    NRAffiliationVocabularyUISchema,
    NRAuthorityRoleVocabularyUISchema,
    NREventUISchema,
    NRExternalLocationUISchema,
    NRFundingReferenceUISchema,
    NRGeoLocationUISchema,
    NRLanguageVocabularyUISchema,
    NRLicenseVocabularyUISchema,
    NRRelatedItemUISchema,
    NRResourceTypeVocabularyUISchema,
    NRSeriesUISchema,
    NRSubjectCategoryVocabularyUISchema,
    NRSubjectUISchema,
)
from nr_metadata.ui_schema.identifiers import (
    NRAuthorityIdentifierUISchema,
    NRObjectIdentifierUISchema,
    NRSystemIdentifierUISchema,
)
from nr_metadata.ui_schema.subjects import NRSubjectListField


class NRCommonRecordUISchema(InvenioUISchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma.fields.Nested(lambda: NRCommonMetadataUISchema())


class NRCommonMetadataUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    abstract = MultilingualUIField(I18nStrUIField())

    accessRights = ma.fields.Nested(lambda: NRAccessRightsVocabularyUISchema())

    accessibility = MultilingualLocalizedUIField(I18nStrUIField())

    additionalTitles = ma.fields.List(
        ma.fields.Nested(lambda: AdditionalTitlesUISchema())
    )

    contributors = ma.fields.List(ma.fields.Nested(lambda: NRContributorUISchema()))

    creators = ma.fields.List(
        ma.fields.Nested(lambda: NRCreatorUISchema()), required=True
    )

    dateAvailable = l10n.LocalizedEDTF()

    dateModified = l10n.LocalizedEDTF()

    events = ma.fields.List(ma.fields.Nested(lambda: NREventUISchema()))

    externalLocation = ma.fields.Nested(lambda: NRExternalLocationUISchema())

    fundingReferences = ma.fields.List(
        ma.fields.Nested(lambda: NRFundingReferenceUISchema())
    )

    geoLocations = ma.fields.List(ma.fields.Nested(lambda: NRGeoLocationUISchema()))

    languages = ma.fields.List(
        ma.fields.Nested(lambda: NRLanguageVocabularyUISchema()), required=True
    )

    methods = MultilingualUIField(I18nStrUIField())

    notes = ma.fields.List(ma.fields.String())

    objectIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRObjectIdentifierUISchema())
    )

    originalRecord = ma.fields.String()

    publishers = ma.fields.List(ma.fields.String())

    relatedItems = ma.fields.List(ma.fields.Nested(lambda: NRRelatedItemUISchema()))

    resourceType = ma.fields.Nested(
        lambda: NRResourceTypeVocabularyUISchema(), required=True
    )

    rights = ma.fields.List(ma.fields.Nested(lambda: NRLicenseVocabularyUISchema()))

    series = ma.fields.List(ma.fields.Nested(lambda: NRSeriesUISchema()))

    subjectCategories = ma.fields.List(
        ma.fields.Nested(lambda: NRSubjectCategoryVocabularyUISchema())
    )

    subjects = NRSubjectListField(ma.fields.Nested(lambda: NRSubjectUISchema()))

    systemIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRSystemIdentifierUISchema())
    )

    technicalInfo = MultilingualUIField(I18nStrUIField())

    title = ma.fields.String(required=True)

    version = ma.fields.String()


class AdditionalTitlesUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    title = I18nStrUIField(required=True)

    titleType = ma.fields.String(
        required=True,
        validate=[
            ma_validate.OneOf(
                ["translatedTitle", "alternativeTitle", "subtitle", "other"]
            )
        ],
    )


class NRContributorUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma.fields.List(
        ma.fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRAuthorityIdentifierUISchema())
    )

    fullName = ma.fields.String(required=True)

    nameType = ma.fields.String(
        validate=[ma_validate.OneOf(["Organizational", "Personal"])]
    )

    role = ma.fields.Nested(lambda: NRAuthorityRoleVocabularyUISchema())


class NRCreatorUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma.fields.List(
        ma.fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRAuthorityIdentifierUISchema())
    )

    fullName = ma.fields.String(required=True)

    nameType = ma.fields.String(
        validate=[ma_validate.OneOf(["Organizational", "Personal"])]
    )



import marshmallow as ma
from marshmallow import validate as ma_validate
from oarepo_runtime.i18n.ui_schema import I18nStrUIField
from oarepo_runtime.ui import marshmallow as l10n
from oarepo_vocabularies.services.ui_schema import (
    HierarchyUISchema,
    VocabularyI18nStrUIField,
)

from nr_metadata.ui_schema.identifiers import (
    NRAuthorityIdentifierUISchema,
    NRObjectIdentifierUISchema,
)


class NREventUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    eventDate = l10n.LocalizedEDTFInterval(required=True)

    eventLocation = ma.fields.Nested(lambda: NRLocationUISchema(), required=True)

    eventNameAlternate = ma.fields.List(ma.fields.String())

    eventNameOriginal = ma.fields.String(required=True)


class NRRelatedItemUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    itemContributors = ma.fields.List(
        ma.fields.Nested(lambda: NRRelatedItemContributorUISchema())
    )

    itemCreators = ma.fields.List(
        ma.fields.Nested(lambda: NRRelatedItemCreatorUISchema())
    )

    itemEndPage = ma.fields.String()

    itemIssue = ma.fields.String()

    itemPIDs = ma.fields.List(ma.fields.Nested(lambda: NRObjectIdentifierUISchema()))

    itemPublisher = ma.fields.String()

    itemRelationType = ma.fields.Nested(lambda: NRItemRelationTypeVocabularyUISchema())

    itemResourceType = ma.fields.Nested(lambda: NRResourceTypeVocabularyUISchema())

    itemStartPage = ma.fields.String()

    itemTitle = ma.fields.String(required=True)

    itemURL = ma.fields.String()

    itemVolume = ma.fields.String()

    itemYear = ma.fields.Integer()


class NRFundingReferenceUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    funder = ma.fields.Nested(lambda: NRFunderVocabularyUISchema())

    fundingProgram = ma.fields.String()

    projectID = ma.fields.String(required=True)

    projectName = ma.fields.String()


class NRGeoLocationUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    geoLocationPlace = ma.fields.String(required=True)

    geoLocationPoint = ma.fields.Nested(lambda: NRGeoLocationPointUISchema())


class NRLocationUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    country = ma.fields.Nested(lambda: NRCountryVocabularyUISchema())

    place = ma.fields.String(required=True)


class NRRelatedItemContributorUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma.fields.List(
        ma.fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRAuthorityIdentifierUISchema())
    )

    fullName = ma.fields.String(required=True)

    nameType = ma.fields.String(
        validate=[ma_validate.OneOf(["Organizational", "Personal"])]
    )

    role = ma.fields.Nested(lambda: NRAuthorityRoleVocabularyUISchema())


class NRRelatedItemCreatorUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma.fields.List(
        ma.fields.Nested(lambda: NRAffiliationVocabularyUISchema())
    )

    authorityIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRAuthorityIdentifierUISchema())
    )

    fullName = ma.fields.String(required=True)

    nameType = ma.fields.String(
        validate=[ma_validate.OneOf(["Organizational", "Personal"])]
    )


class NRAccessRightsVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRAffiliationVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    hierarchy = ma.fields.Nested(lambda: HierarchyUISchema())

    title = VocabularyI18nStrUIField()


class NRAuthorityRoleVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRCountryVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRExternalLocationUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    externalLocationNote = ma.fields.String()

    externalLocationURL = ma.fields.String(required=True)


class NRFunderVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRGeoLocationPointUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    pointLatitude = ma.fields.Float(required=True)

    pointLongitude = ma.fields.Float(required=True)


class NRItemRelationTypeVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRLanguageVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRLicenseVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRResourceTypeVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRSeriesUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    seriesTitle = ma.fields.String(required=True)

    seriesVolume = ma.fields.String()


class NRSubjectCategoryVocabularyUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRSubjectUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    classificationCode = ma.fields.String()

    subject = I18nStrUIField()

    subjectScheme = ma.fields.String()

    valueURI = ma.fields.String()



import marshmallow as ma
from edtf import Interval as EDTFInterval
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import validate as ma_validate
from marshmallow_utils.fields import TrimmedString
from oarepo_runtime.i18n.schema import I18nStrField, MultilingualField
from oarepo_runtime.validation import CachedMultilayerEDTFValidator
from oarepo_vocabularies.services.schema import HierarchySchema

from nr_metadata.schema.identifiers import (
    NRAuthorityIdentifierSchema,
    NRObjectIdentifierSchema,
)


class NREventSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    eventDate = TrimmedString(
        required=True, validate=[CachedMultilayerEDTFValidator(types=(EDTFInterval,))]
    )

    eventLocation = ma.fields.Nested(lambda: NRLocationSchema(), required=True)

    eventNameAlternate = ma.fields.List(ma.fields.String())

    eventNameOriginal = ma.fields.String(required=True)


class NRRelatedItemSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    itemContributors = ma.fields.List(
        ma.fields.Nested(lambda: NRRelatedItemContributorSchema())
    )

    itemCreators = ma.fields.List(
        ma.fields.Nested(lambda: NRRelatedItemCreatorSchema())
    )

    itemEndPage = ma.fields.String()

    itemIssue = ma.fields.String()

    itemPIDs = ma.fields.List(ma.fields.Nested(lambda: NRObjectIdentifierSchema()))

    itemPublisher = ma.fields.String()

    itemRelationType = ma.fields.Nested(lambda: NRItemRelationTypeVocabularySchema())

    itemResourceType = ma.fields.Nested(lambda: NRResourceTypeVocabularySchema())

    itemStartPage = ma.fields.String()

    itemTitle = ma.fields.String(required=True)

    itemURL = ma.fields.String()

    itemVolume = ma.fields.String()

    itemYear = ma.fields.Integer()


class NRFundingReferenceSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    funder = ma.fields.Nested(lambda: NRFunderVocabularySchema())

    fundingProgram = ma.fields.String()

    projectID = ma.fields.String(required=True)

    projectName = ma.fields.String()


class NRGeoLocationSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    geoLocationPlace = ma.fields.String(required=True)

    geoLocationPoint = ma.fields.Nested(lambda: NRGeoLocationPointSchema())


class NRLocationSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    country = ma.fields.Nested(lambda: NRCountryVocabularySchema())

    place = ma.fields.String(required=True)


class NRRelatedItemContributorSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma.fields.List(
        ma.fields.Nested(lambda: NRAffiliationVocabularySchema())
    )

    authorityIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRAuthorityIdentifierSchema())
    )

    fullName = ma.fields.String(required=True)

    nameType = ma.fields.String(
        validate=[ma_validate.OneOf(["Organizational", "Personal"])]
    )

    role = ma.fields.Nested(lambda: NRAuthorityRoleVocabularySchema())


class NRRelatedItemCreatorSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    affiliations = ma.fields.List(
        ma.fields.Nested(lambda: NRAffiliationVocabularySchema())
    )

    authorityIdentifiers = ma.fields.List(
        ma.fields.Nested(lambda: NRAuthorityIdentifierSchema())
    )

    fullName = ma.fields.String(required=True)

    nameType = ma.fields.String(
        validate=[ma_validate.OneOf(["Organizational", "Personal"])]
    )


class NRAccessRightsVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRAffiliationVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    hierarchy = ma.fields.Nested(lambda: HierarchySchema())

    title = i18n_strings


class NRAuthorityRoleVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRCountryVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRExternalLocationSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    externalLocationNote = ma.fields.String()

    externalLocationURL = ma.fields.String(required=True)


class NRFunderVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRGeoLocationPointSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    pointLatitude = ma.fields.Float(
        required=True, validate=[ma.validate.Range(min=-90.0, max=90.0)]
    )

    pointLongitude = ma.fields.Float(
        required=True, validate=[ma.validate.Range(min=-180.0, max=180.0)]
    )


class NRItemRelationTypeVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRLanguageVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRLicenseVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRResourceTypeVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRSeriesSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    seriesTitle = ma.fields.String(required=True)

    seriesVolume = ma.fields.String()


class NRSubjectCategoryVocabularySchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRSubjectSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    classificationCode = ma.fields.String()

    subject = MultilingualField(I18nStrField(), required=True)

    subjectScheme = ma.fields.String()

    valueURI = ma.fields.String()



import marshmallow as ma
from invenio_vocabularies.services.schema import i18n_strings
from oarepo_runtime.marshmallow import BaseRecordSchema
from oarepo_runtime.validation import validate_date
from oarepo_vocabularies.services.schema import HierarchySchema

import nr_metadata.common.services.records.schema_common
import nr_metadata.common.services.records.schema_datatypes
import nr_metadata.schema.identifiers


class NRDocumentRecordSchema(BaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma.fields.Nested(lambda: NRDocumentMetadataSchema())

    syntheticFields = ma.fields.Nested(lambda: SyntheticFieldsSchema())


class NRDocumentMetadataSchema(
    nr_metadata.common.services.records.schema_common.NRCommonMetadataSchema
):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma.fields.List(
        ma.fields.Nested(lambda: AdditionalTitlesItemSchema())
    )

    collection = ma.fields.String()

    contributors = ma.fields.List(ma.fields.Nested(lambda: ContributorsItemSchema()))

    creators = ma.fields.List(
        ma.fields.Nested(lambda: CreatorsItemSchema()),
        required=True,
        validate=[ma.validate.Length(min=1)],
    )

    thesis = ma.fields.Nested(lambda: NRThesisSchema())


class GeoLocationsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRGeoLocationSchema
):
    class Meta:
        unknown = ma.RAISE

    geoLocationPoint = ma.fields.Nested(lambda: GeoLocationPointSchema())


class NRThesisSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    dateDefended = ma.fields.String(validate=[validate_date("%Y-%m-%d")])

    defended = ma.fields.Boolean()

    degreeGrantors = ma.fields.List(ma.fields.Nested(lambda: NRDegreeGrantorSchema()))

    studyFields = ma.fields.List(ma.fields.String())


class RelatedItemsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRRelatedItemSchema
):
    class Meta:
        unknown = ma.RAISE

    itemContributors = ma.fields.List(
        ma.fields.Nested(lambda: ItemContributorsItemSchema())
    )

    itemCreators = ma.fields.List(ma.fields.Nested(lambda: ItemCreatorsItemSchema()))


class AccessRightsSchema(
    nr_metadata.common.services.records.schema_datatypes.NRAccessRightsVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class AdditionalTitlesItemSchema(
    nr_metadata.common.services.records.schema_common.AdditionalTitlesSchema
):
    class Meta:
        unknown = ma.RAISE


class AffiliationsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRAffiliationVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    hierarchy = ma.fields.Nested(lambda: HierarchySchema())

    title = i18n_strings


class AuthorityIdentifiersItemSchema(
    nr_metadata.schema.identifiers.NRAuthorityIdentifierSchema
):
    class Meta:
        unknown = ma.RAISE


class ContributorsItemSchema(
    nr_metadata.common.services.records.schema_common.NRContributorSchema
):
    class Meta:
        unknown = ma.RAISE


class CountrySchema(
    nr_metadata.common.services.records.schema_datatypes.NRCountryVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class CreatorsItemSchema(
    nr_metadata.common.services.records.schema_common.NRCreatorSchema
):
    class Meta:
        unknown = ma.RAISE


class EventLocationSchema(
    nr_metadata.common.services.records.schema_datatypes.NRLocationSchema
):
    class Meta:
        unknown = ma.RAISE


class EventsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NREventSchema
):
    class Meta:
        unknown = ma.RAISE


class ExternalLocationSchema(
    nr_metadata.common.services.records.schema_datatypes.NRExternalLocationSchema
):
    class Meta:
        unknown = ma.RAISE


class FunderSchema(
    nr_metadata.common.services.records.schema_datatypes.NRFunderVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class FundingReferencesItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRFundingReferenceSchema
):
    class Meta:
        unknown = ma.RAISE


class GeoLocationPointSchema(
    nr_metadata.common.services.records.schema_datatypes.NRGeoLocationPointSchema
):
    class Meta:
        unknown = ma.RAISE


class ItemContributorsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRRelatedItemContributorSchema
):
    class Meta:
        unknown = ma.RAISE


class ItemCreatorsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRRelatedItemCreatorSchema
):
    class Meta:
        unknown = ma.RAISE


class ItemRelationTypeSchema(
    nr_metadata.common.services.records.schema_datatypes.NRItemRelationTypeVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class ItemResourceTypeSchema(
    nr_metadata.common.services.records.schema_datatypes.NRResourceTypeVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class LanguagesItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRLanguageVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRDegreeGrantorSchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    hierarchy = ma.fields.Nested(lambda: HierarchySchema())

    title = i18n_strings


class ObjectIdentifiersItemSchema(
    nr_metadata.schema.identifiers.NRObjectIdentifierSchema
):
    class Meta:
        unknown = ma.RAISE


class RightsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRLicenseVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class RoleSchema(
    nr_metadata.common.services.records.schema_datatypes.NRAuthorityRoleVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class SeriesItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRSeriesSchema
):
    class Meta:
        unknown = ma.RAISE


class SubjectCategoriesItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRSubjectCategoryVocabularySchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = i18n_strings


class SubjectsItemSchema(
    nr_metadata.common.services.records.schema_datatypes.NRSubjectSchema
):
    class Meta:
        unknown = ma.RAISE


class SyntheticFieldsSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE


class SystemIdentifiersItemSchema(
    nr_metadata.schema.identifiers.NRSystemIdentifierSchema
):
    class Meta:
        unknown = ma.RAISE



import marshmallow as ma
from oarepo_runtime.ui import marshmallow as l10n
from oarepo_runtime.ui.marshmallow import InvenioUISchema
from oarepo_vocabularies.services.ui_schema import (
    HierarchyUISchema,
    VocabularyI18nStrUIField,
)

import nr_metadata.common.services.records.ui_schema_common
import nr_metadata.common.services.records.ui_schema_datatypes
import nr_metadata.ui_schema.identifiers


class NRDocumentRecordUISchema(InvenioUISchema):
    class Meta:
        unknown = ma.RAISE

    metadata = ma.fields.Nested(lambda: NRDocumentMetadataUISchema())

    syntheticFields = ma.fields.Nested(lambda: SyntheticFieldsUISchema())


class NRDocumentMetadataUISchema(
    nr_metadata.common.services.records.ui_schema_common.NRCommonMetadataUISchema
):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma.fields.List(
        ma.fields.Nested(lambda: AdditionalTitlesItemUISchema())
    )

    collection = ma.fields.String()

    contributors = ma.fields.List(ma.fields.Nested(lambda: ContributorsItemUISchema()))

    creators = ma.fields.List(
        ma.fields.Nested(lambda: CreatorsItemUISchema()), required=True
    )

    thesis = ma.fields.Nested(lambda: NRThesisUISchema())


class GeoLocationsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRGeoLocationUISchema
):
    class Meta:
        unknown = ma.RAISE

    geoLocationPoint = ma.fields.Nested(lambda: GeoLocationPointUISchema())


class NRThesisUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    dateDefended = l10n.LocalizedDate()

    defended = ma.fields.Boolean()

    degreeGrantors = ma.fields.List(ma.fields.Nested(lambda: NRDegreeGrantorUISchema()))

    studyFields = ma.fields.List(ma.fields.String())


class RelatedItemsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRRelatedItemUISchema
):
    class Meta:
        unknown = ma.RAISE

    itemContributors = ma.fields.List(
        ma.fields.Nested(lambda: ItemContributorsItemUISchema())
    )

    itemCreators = ma.fields.List(ma.fields.Nested(lambda: ItemCreatorsItemUISchema()))


class AccessRightsUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRAccessRightsVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class AdditionalTitlesItemUISchema(
    nr_metadata.common.services.records.ui_schema_common.AdditionalTitlesUISchema
):
    class Meta:
        unknown = ma.RAISE


class AffiliationsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRAffiliationVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    hierarchy = ma.fields.Nested(lambda: HierarchyUISchema())

    title = VocabularyI18nStrUIField()


class AuthorityIdentifiersItemUISchema(
    nr_metadata.ui_schema.identifiers.NRAuthorityIdentifierUISchema
):
    class Meta:
        unknown = ma.RAISE


class ContributorsItemUISchema(
    nr_metadata.common.services.records.ui_schema_common.NRContributorUISchema
):
    class Meta:
        unknown = ma.RAISE


class CountryUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRCountryVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class CreatorsItemUISchema(
    nr_metadata.common.services.records.ui_schema_common.NRCreatorUISchema
):
    class Meta:
        unknown = ma.RAISE


class EventLocationUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRLocationUISchema
):
    class Meta:
        unknown = ma.RAISE


class EventsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NREventUISchema
):
    class Meta:
        unknown = ma.RAISE


class ExternalLocationUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRExternalLocationUISchema
):
    class Meta:
        unknown = ma.RAISE


class FunderUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRFunderVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class FundingReferencesItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRFundingReferenceUISchema
):
    class Meta:
        unknown = ma.RAISE


class GeoLocationPointUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRGeoLocationPointUISchema
):
    class Meta:
        unknown = ma.RAISE


class ItemContributorsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRRelatedItemContributorUISchema
):
    class Meta:
        unknown = ma.RAISE


class ItemCreatorsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRRelatedItemCreatorUISchema
):
    class Meta:
        unknown = ma.RAISE


class ItemRelationTypeUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRItemRelationTypeVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class ItemResourceTypeUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRResourceTypeVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class LanguagesItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRLanguageVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRDegreeGrantorUISchema(ma.Schema):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    hierarchy = ma.fields.Nested(lambda: HierarchyUISchema())

    title = VocabularyI18nStrUIField()


class ObjectIdentifiersItemUISchema(
    nr_metadata.ui_schema.identifiers.NRObjectIdentifierUISchema
):
    class Meta:
        unknown = ma.RAISE


class RightsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRLicenseVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class RoleUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRAuthorityRoleVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class SeriesItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRSeriesUISchema
):
    class Meta:
        unknown = ma.RAISE


class SubjectCategoriesItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRSubjectCategoryVocabularyUISchema
):
    class Meta:
        unknown = ma.INCLUDE

    _id = ma.fields.String(data_key="id", attribute="id")

    _version = ma.fields.String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class SubjectsItemUISchema(
    nr_metadata.common.services.records.ui_schema_datatypes.NRSubjectUISchema
):
    class Meta:
        unknown = ma.RAISE


class SyntheticFieldsUISchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    institutions = ma.fields.String()

    keywords_cs = ma.fields.String()

    keywords_en = ma.fields.String()

    person = ma.fields.String()


class SystemIdentifiersItemUISchema(
    nr_metadata.ui_schema.identifiers.NRSystemIdentifierUISchema
):
    class Meta:
        unknown = ma.RAISE
